# shellcheck disable=SC2148
VENDOR="${VENDOR:-solarwinds}"

DEFAULT_INSTALL_DIR="/opt/SolarWinds/Snap"
INSTALL_DIR=${INSTALL_DIR:-${DEFAULT_INSTALL_DIR}}
# shellcheck disable=SC2034
AGENT_CONF="${INSTALL_DIR}/etc/config.yaml"
LOGS_PLUGIN_CONFIG="${INSTALL_DIR}/etc/plugins.d/logs.yaml"
# shellcheck disable=SC2034
LOGS_PLUGIN_CONFIG_EXAMPLE="${LOGS_PLUGIN_CONFIG}.example"

# shellcheck disable=SC2034
AGENT_NAME="swisnapd"
# shellcheck disable=SC2034
DAEMON_NAME="swisnapd"
# shellcheck disable=SC2034
AGENT_DISPLAY_NAME="SolarWinds Snap Agent"

# shellcheck disable=SC2034
AGENT_TMP_DIR="/tmp/SolarWinds/Snap"
# shellcheck disable=SC2034
AGENT_LOG_DIR="/var/log/SolarWinds/Snap"
# shellcheck disable=SC2034
PID_DIR="/var/run/${VENDOR}"
# shellcheck disable=SC2034
SYMLINK_DIR=/usr/local/bin
# shellcheck disable=SC2034
LOG_ROTATION_CONF_DIR=/etc/logrotate.d

# shellcheck disable=SC2034
SYSTEMCTL_BIN=$(command -v systemctl 2>/dev/null)
# shellcheck disable=SC2034
CHKCONFIG_BIN=$(command -v chkconfig 2>/dev/null)
# shellcheck disable=SC2034
SERVICE_BIN=$(command -v service 2>/dev/null)
# shellcheck disable=SC2034
DPKG_QUERY_BIN=$(command -v dpkg-query 2>/dev/null)
# shellcheck disable=SC2034
LOGROTATE_BIN=$(command -v logrotate 2>/dev/null)
# shellcheck disable=SC2034
UPGRADE_SAVE_TOKEN_FILE="${INSTALL_DIR}/etc/upgrade.token.backup"
# shellcheck disable=SC2034
RM_CMD=/bin/rm

# Settings for the user used to run the daemon
# shellcheck disable=SC2034
AGENT_USERNAME=${VENDOR}
# shellcheck disable=SC2034
AGENT_GROUPNAME=${VENDOR}

# truth values
__IS_TRUE=1
__IS_FALSE=0

# When set to true, install the package but don't start the service
AGENT_INSTALL_ONLY=${AGENT_INSTALL_ONLY:-${APPOPTICS_INSTALL_ONLY:-${__IS_FALSE}}}

# Contains the current installation step of the post installer
# shellcheck disable=SC2034
__step=1

_ECHO_DEBUG=${_ECHO_DEBUG:-${__IS_FALSE}}
_SD_NO_COLOR=${_SD_NO_COLOR:-${__IS_TRUE}}

# shellcheck disable=SC2034
ROOT_DIR=$(pwd)

# Return error codes
readonly SUCCESS=0
readonly FAILURE=1
readonly FAILURE_SERVICE_NOT_RUNNING=2
readonly FAILURE_NOT_ENOUGH_RIGHTS=3
readonly FAILURE_UNABLE_TO_EXTRACT=4
readonly FAILURE_MISSING_EXECUTABLE=5
readonly FAILURE_CREATE_AGENT_GROUP=6
readonly FAILURE_CREATE_AGENT_USER=7
readonly FAILURE_SET_ACCESS_RIGHTS_AGENT_USER=8
_COLORS=${BS_COLORS:-$(tput colors 2>/dev/null || echo 0)}
detect_color_support() {
    # shellcheck disable=SC2181
    if [ $? -eq 0 ] && [ "$_COLORS" -gt 2 ] && [ "$_SD_NO_COLOR" -eq "$__IS_FALSE" ]; then
        RC="\033[1;31m"
        GC="\033[1;32m"
        BC="\033[1;34m"
        YC="\033[1;33m"
        EC="\033[0m"
    else
        RC=""
        GC=""
        BC=""
        YC=""
        EC=""
    fi
}
detect_color_support

# Override built-in echo for consistency. Has newline at end
echo() (
    fmt=%s end=\\n IFS=" "

    while [ $# -gt 1 ]; do
        case "$1" in
        [!-]* | -*[!ne]*) break ;;
        *ne* | *en*) fmt=%b end= ;;
        *n*) end= ;;
        *e*) fmt=%b ;;
        esac
        shift
    done

    # shellcheck disable=SC2059
    printf "$fmt$end" "$*"
)

echored() {
    printf "${RC}%s${EC}\n" "$*"
}

echogreen() {
    printf "${GC}%s${EC}\n" "$*"
}

echoblue() {
    printf "${BC}%s${EC}\n" "$*"
}

echoyellow() {
    printf "${YC}%s${EC}\n" "$*"
}
__increment_step() {
    __step=$((__step + 1))
}

exists() {
    command -v "$1" >/dev/null 2>&1
}

now() {
    date +'%Y-%m-%d-%H_%M_%S'
}

create_install_directories() {
    echodebug "Creating support directories ..."
    mkdir -p "${AGENT_TMP_DIR}"
    mkdir -p "${AGENT_LOG_DIR}"
    mkdir -p "${PID_DIR}"
    echodebug "directories created."
}

get_package_version() {
    local version=
    if [ -x "${DPKG_QUERY_BIN}" ]; then
        version="$(dpkg-query -W solarwinds-snap-agent)"
    else
        version="$(rpm -q solarwinds-snap-agent)"
    fi
    echo "${version}"
}

set_swoken() {
    if [ -n "${SOLARWINDS_TOKEN}" ]; then
        SOLARWINDS_TOKEN=${SOLARWINDS_TOKEN}
    elif [ -n "${APPOPTICS_TOKEN}" ]; then
        SOLARWINDS_TOKEN=${APPOPTICS_TOKEN}
    else
        SOLARWINDS_TOKEN="SOLARWINDS_TOKEN"
    fi
}

insert_appoptics_url() {
    sed -i "s,APPOPTICS_URL,\"${APPOPTICS_URL}\",g" "${AGENT_CONF}"
    sed -i "s,APPOPTICS_REPORT_URL,\"${APPOPTICS_REPORT_URL}\",g" "${AGENT_CONF}"
}
echoerror() {
    printf "${RC} * ERROR${EC}: %s\n" "$*" 1>&2
}

info() {
    printf "${GC} *  INFO${EC}: %s\n" "$*"
}

warning() {
    printf "${YC} *  WARN${EC}: %s\n" "$*"
}

echodebug() {
    if [ "$_ECHO_DEBUG" = "$__IS_TRUE" ]; then
        printf "${BC} * DEBUG${EC}: %s\n" "$*"
    fi
}

error() {
    error_code=${1}
    error_description=${2:-""}
    error_date_time="$(now)"
    echoerror "######################################################"
    echoerror "An error has occured during uninstallation of the agent."
    echoerror
    echoerror "Description: ${error_description}"
    echoerror "Error code : ${error_code}"
    echoerror "When       : ${error_date_time}"
    echoerror "######################################################"
}

failed() {
    error_code=${1}
    echo -e "${BC}===============${RC} FAILED ${BC}======================${EC}"
    echo -e "${BC}An error occurred during installation of ${RC}${AGENT_DISPLAY_NAME}${BC}.${EC}"
    echo -e "${BC}Please contact AppOptics customer support at ${RC}support@appoptics.com${BC}.${EC}"
    echo -e "${BC}Error code: ${RC}${error_code}${BC}.${EC}"
    echo -e "${BC}========================================================${EC}"
}
create_daemon_user() {
    info "(${__step}) Configuring user and group settings ..."
    __increment_step

    # Check if the group already exists
    if getent group "${AGENT_GROUPNAME}" >/dev/null; then
        info "Group ${AGENT_GROUPNAME} already exists. No need to create it."
    else
        info "Creating ${AGENT_GROUPNAME} system group ..."
        groupadd --system "${AGENT_GROUPNAME}"
        RETVAL=$?
        if [ $RETVAL -ne 0 ]; then
            echoerror "Unable to create ${AGENT_GROUPNAME} system group. Error code: ${RETVAL}."
            return "${FAILURE_CREATE_AGENT_GROUP}"
        fi
        info "${AGENT_GROUPNAME} group created."
    fi

    if ! getent passwd "${AGENT_USERNAME}" >/dev/null; then
        info "Creating ${AGENT_USERNAME} system user ..."
        useradd --system \
            --gid "${AGENT_GROUPNAME}" \
            --no-create-home \
            --home-dir "${INSTALL_DIR}" \
            --comment "${AGENT_DISPLAY_NAME} Daemon" \
            --shell /bin/sh \
            "${AGENT_USERNAME}"
        RETVAL=$?
        if [ $RETVAL -ne 0 ]; then
            echoerror "Unable to create ${AGENT_USERNAME} system user. Error code: ${RETVAL}."
            return "${FAILURE_CREATE_AGENT_USER}"
        fi
        info "${AGENT_USERNAME} system user created."
    else
        info "System user ${AGENT_USERNAME} already exists. No need to create it."
    fi

    echodebug "user and group settings configured."
    return "${SUCCESS}"
}

set_daemon_user_rights() {
    info "(${__step}) Setting user access rights ..."
    __increment_step

    ############################
    # General user requirements

    # Changed user to be owner of the installation directory
    chown "${AGENT_USERNAME}":"${AGENT_GROUPNAME}" -R "${INSTALL_DIR}"

    # Changed user to be owner of the temporary directory.
    # Temporary directory is used by daemon to launch executables,
    # logging.
    chown "${AGENT_USERNAME}":"${AGENT_GROUPNAME}" -R "${AGENT_TMP_DIR}"
    chown "${AGENT_USERNAME}":"${AGENT_GROUPNAME}" -R "${AGENT_LOG_DIR}"
    chown "${AGENT_USERNAME}":"${AGENT_GROUPNAME}" -R "${PID_DIR}"
    chmod 775 -R "${AGENT_TMP_DIR}"
    chmod 775 -R "${AGENT_LOG_DIR}"
    chmod 775 -R "${PID_DIR}"
    chmod 640 "${AGENT_CONF}"

    # Add user to Docker and adm groups (required by docker and logs plugins)
    groupadd_plugin_requirement docker Docker
    groupadd_plugin_requirement adm Logs

    echodebug "user access rights configured."
    return "${SUCCESS}"
}

groupadd_plugin_requirement() {
    local group=$1
    local plugin=$2
    if getent group "${group}" >/dev/null; then
        info "Found ${group} group. Adding ${AGENT_USERNAME} to ${group} group ..."
        usermod -aG "${group}" "${AGENT_USERNAME}"
        RETVAL=$?
        if [ $RETVAL -ne 0 ]; then
            echoerror "Unable to add ${AGENT_USERNAME} user to ${group} group. Error code: ${RETVAL}."
            return "${FAILURE_SET_ACCESS_RIGHTS_AGENT_USER}"
        else
            info "${AGENT_USERNAME} added to ${group} group."
        fi
    else
        warning "Unable to find ${group} group. ${plugin} plugin will not work without ${AGENT_USERNAME} user being added to ${group} group."
    fi
}
running_in_docker() {
    if grep -q "docker" /proc/self/cgroup; then
        return 1
    fi
    return 0
}

is_process_running() {
    ps -C "${1}" >/dev/null 2>&1
}

daemon_running() {
    is_process_running swisnapd
}

any_plugins_running() {
    pgrep -l 'snap-plugin' >/dev/null 2>&1
}

kill_agent() {
    if daemon_running; then
        echodebug "Killing ${AGENT_DISPLAY_NAME} service ..."
        pkill -TERM -U "${AGENT_USERNAME}" swisnapd
        sleep 1
        pkill -KILL -U "${AGENT_USERNAME}" swisnapd
    fi

    if any_plugins_running; then
        echodebug "Killing ${AGENT_NAME} and all child processes ..."
        pkill -KILL -P "$(pgrep "${AGENT_NAME}")" -U "${AGENT_USERNAME}"
        pkill -KILL "${AGENT_NAME}" -U "${AGENT_USERNAME}"
    fi
}

stop_service() {
    echodebug "Stopping ${AGENT_DISPLAY_NAME} service ..."
    if [ -x "${SYSTEMCTL_BIN}" ]; then
        # Stop service
        ${SYSTEMCTL_BIN} stop "${AGENT_NAME}"
    else
        # Stop service
        ${SERVICE_BIN} "${AGENT_NAME}" stop
    fi

    kill_agent
}
install_service() {
    # Check if daemon and processes are running from a previous install
    # Some older versions are failing to properly stop the service,
    # so stop service before installing a new package.
    if daemon_running; then
        echodebug "Detected ${AGENT_DISPLAY_NAME} is running. Stop ${AGENT_DISPLAY_NAME} before installing ..."
        stop_service
    fi

    chmod 775 "${INSTALL_DIR}"/bin/*
    chmod 775 "${INSTALL_DIR}"/sbin/*

    info "(${__step}) Checking if system uses systemd or SysV(/etc/init.d) to manage the services."
    __increment_step

    # Check if system uses systemd
    if [ -x "${SYSTEMCTL_BIN}" ]; then
        info "Detected that system uses systemd to manage the services."
        if [ ! -w /lib/systemd/system ]; then
            RETVAL=${FAILURE_NOT_ENOUGH_RIGHTS}
            error "${RETVAL}" "Not enough user rights. Please run the script as superuser or check the rights on /lib/systemd/system directory."
            return "$RETVAL"
        else
            echodebug "Copying service files and configuration required by systemd ..."
            # Install script for a system that supports systemd
            cp "${INSTALL_DIR}/support/${AGENT_NAME}.service" "/lib/systemd/system/${AGENT_NAME}.service"

            # Copy file containing service environment variables
            if [ -d /etc/default ]; then
                cp "${INSTALL_DIR}/support/swisnapd.sysconfig" "/etc/default/swisnapd"
            else
                if [ -d /etc/sysconfig ]; then
                    cp "${INSTALL_DIR}/support/swisnapd.sysconfig" "/etc/sysconfig/swisnapd"
                else
                    warning "Can't copy the ${AGENT_NAME} enviroment file."
                fi
            fi

            # Reload daemon configurations
            info "(${__step}) Reloading configuration for ${AGENT_NAME} service ..."
            __increment_step

            ${SYSTEMCTL_BIN} daemon-reload
            echodebug "${AGENT_NAME} service reloaded."

            # Enable service to be executed at boot time
            info "(${__step}) Enabling ${AGENT_NAME} service to be loaded at boot time ..."
            __increment_step
            ${SYSTEMCTL_BIN} enable "${AGENT_NAME}.service"

            # Check if service is enabled
            if [ "$(${SYSTEMCTL_BIN} is-enabled "${AGENT_NAME}")" = "enabled" ]; then
                echodebug "Service ${AGENT_NAME} has been enabled to load at boot time."
            else
                warning "Unable to enable service ${AGENT_NAME}."
            fi

            # Check to see if running inside a docker with systemd
            can_start_service=${__IS_TRUE}
            if ! running_in_docker; then
                echodebug "Running inside a Docker container with systemd."
                can_start_service=${__IS_FALSE}
            fi

            RETVAL=${SUCCESS}

            if [ "${can_start_service}" -eq "${__IS_TRUE}" ] && [ "${AGENT_INSTALL_ONLY}" -eq "${__IS_FALSE}" ]; then
                # Start service
                info "(${__step}) Starting ${AGENT_NAME} service ..."
                __increment_step
                if ${SYSTEMCTL_BIN} start "${AGENT_NAME}.service"; then
                    echodebug "${AGENT_NAME} service has STARTED."
                else
                    RETVAL=${FAILURE_SERVICE_NOT_RUNNING}
                    error "${RETVAL}" "Unable to start ${AGENT_NAME} service. Service is NOT STARTED."
                fi

                # Wait for 10 seconds for service to be fully initialized.
                # Avoids case when service is running for a couple of seconds and fails because of configuration
                sleep 10

                # Check if service is running
                info "(${__step}) Checking if ${AGENT_NAME} service is running ..."
                __increment_step
                if [ "$(${SYSTEMCTL_BIN} is-active "${AGENT_NAME}")" = "active" ]; then
                    echodebug "${AGENT_NAME} service is RUNNING."
                    RETVAL=${SUCCESS}
                else
                    RETVAL=${FAILURE_SERVICE_NOT_RUNNING}
                    error "${RETVAL}" "Unable to start ${AGENT_NAME} service. Service is NOT RUNNING."
                fi
            fi

            return "$RETVAL"
        fi
    else
        # System uses SysV(/etc/init.d)
        info "Detected that system uses SysV(/etc/init.d) to manage the services."
        if [ ! -w /etc/init.d ]; then
            RETVAL=${FAILURE_NOT_ENOUGH_RIGHTS}
            error "${RETVAL}" "Not enough user rights. Please run the script as superuser or check the rights on /etc/init.d directory."
            return "$RETVAL"
        else
            echodebug "Copying service files and configuration required by SysV ..."
            if grep -q Amazon /etc/issue >/dev/null || [ -f /etc/redhat-release ]; then
                # Copy file containing service environment variables
                if [ -d /etc/sysconfig ]; then
                    cp "${INSTALL_DIR}/support/swisnapd.sysconfig" "/etc/sysconfig/swisnapd"
                else
                    warning "Can't copy the ${AGENT_NAME} enviroment file."
                fi

                # Install script for a RedHat based system
                cp "${INSTALL_DIR}/support/${AGENT_NAME}.rh.initd" "/etc/init.d/${AGENT_NAME}"
                chmod +x "/etc/init.d/${AGENT_NAME}"
            else
                # Copy file containing service environment variables
                if [ -d /etc/default ]; then
                    cp "${INSTALL_DIR}/support/swisnapd.sysconfig" "/etc/default/swisnapd"
                else
                    warning "Can't copy the ${AGENT_NAME} enviroment file."
                fi

                # Install script for a Debian based system
                cp "${INSTALL_DIR}/support/${AGENT_NAME}.deb.initd" "/etc/init.d/${AGENT_NAME}"
                chmod +x "/etc/init.d/${AGENT_NAME}"
            fi

            info "(${__step}) Enabling ${AGENT_NAME} service to be loaded at boot time..."
            __increment_step
            # Enable service to be executed at boot time
            if [ -x "${CHKCONFIG_BIN}" ]; then
                ${CHKCONFIG_BIN} "${AGENT_NAME}" on
            else
                /usr/sbin/update-rc.d "${AGENT_NAME}" defaults
            fi
            info "${AGENT_NAME} service ENABLED to load at boot time."

            RETVAL=${SUCCESS}
            if [ "${AGENT_INSTALL_ONLY}" -eq "${__IS_FALSE}" ]; then
                info "(${__step}) Starting ${AGENT_NAME} service ..."
                __increment_step
                # Start service
                if exists "${SERVICE_BIN}"; then
                    if ${SERVICE_BIN} "${AGENT_NAME}" start; then
                        echodebug "${AGENT_NAME} service is STARTED."
                    else
                        RETVAL=${FAILURE_SERVICE_NOT_RUNNING}
                        error "${RETVAL}" "Unable to start ${AGENT_NAME} service. Service is NOT RUNNING."
                    fi
                else
                    RETVAL=${FAILURE_MISSING_EXECUTABLE}
                    error "${RETVAL}" "Unable to start ${AGENT_NAME} service. Service ${SERVICE_BIN} command is missing."
                fi
            fi

            return "$RETVAL"
        fi
    fi
}
create_symlinks_tools() {
    echodebug "Creating symlinks for tools ..."

    local symlink_dir=/usr/local/bin
    mkdir -p ${symlink_dir}

    tool="swisnap"
    symlink_file=${symlink_dir}/${tool}
    source_file=${INSTALL_DIR}/bin/${tool}

    # Delete symlink if already exists
    if [ -e "${symlink_file}" ]; then
        echodebug "Unlink existing ${symlink_file} link."
        rm ${symlink_file}
    fi

    echodebug "Creating symlink ${symlink_file} to ${source_file}..."
    ln -s "${source_file}" "${symlink_file}"

    # Check that symlink was created
    if [ -e "${symlink_file}" ]; then
        echodebug "${symlink_file} created."
    else
        echowarn "Unable to create symlink ${symlink_file} to ${source_file}."
    fi

    echodebug "symlinks created."
}
configure_log_rotation() {
    if [ -x "${LOGROTATE_BIN}" ]; then
        if [ -d "${LOG_ROTATION_CONF_DIR}" ]; then
            # Delete symlink if already exists
            if [ -e ${LOG_ROTATION_CONF_DIR}/solarwinds-snap-agent ]; then
                echodebug "Unlink existing ${LOG_ROTATION_CONF_DIR}/solarwinds-snap-agent link."
                rm ${LOG_ROTATION_CONF_DIR}/solarwinds-snap-agent
            fi

            if grep -iq amazon /etc/system-release 2>/dev/null; then
                sed -i '/su solarwinds solarwinds/d' "${INSTALL_DIR}"/etc/logrotate.d/solarwinds-snap-agent
            fi

            # Create symlink to the logrotate configuration file
            ln -s "${INSTALL_DIR}"/etc/logrotate.d/solarwinds-snap-agent "${LOG_ROTATION_CONF_DIR}"/solarwinds-snap-agent

            # Check if symbolic link has been created
            if [ -e "${LOG_ROTATION_CONF_DIR}/solarwinds-snap-agent" ]; then
                echodebug "Symlink ${LOG_ROTATION_CONF_DIR}/solarwinds-snap-agent created."

                echodebug "Change owner for ${LOG_ROTATION_CONF_DIR}/solarwinds-snap-agent to root."
                chown root:root "${INSTALL_DIR}"/etc/logrotate.d/solarwinds-snap-agent
                chown -h root:root "${LOG_ROTATION_CONF_DIR}"/solarwinds-snap-agent
            else
                echodebug "Unable to create symlink ${LOG_ROTATION_CONF_DIR}/solarwinds-snap-agent."
            fi
        else
            warning "Unable to find ${LOG_ROTATION_CONF_DIR} directory. Log rotation will not work without copying ${INSTALL_DIR}/etc/logrotate.d/${AGENT_NAME} to logrotate configuration directory."
        fi
    else
        echodebug "Could not find logrotate."
    fi
}
# after-upgrade

APPOPTICS_URL=${APPOPTICS_URL:-"https://api.appoptics.com/v1/measurements"}
APPOPTICS_REPORT_URL="https://api.appoptics.com/v1/agent/report"

failed() {
    echored "###################################################"
    echored "An error occurred during the installation of ${AGENT_DISPLAY_NAME}."
    echored "Please contact AppOptics customer service for support at support@appoptics.com."
    echored "###################################################"
}

check_token() {
    # Get the current token saved into configuration file
    current_token="$(grep token "${AGENT_CONF}" | awk '{gsub(/"/, "", $2); print $2}' | head -1)"
    previous_token="$(cat "${UPGRADE_SAVE_TOKEN_FILE}")"

    # Check if token has been overwritten during upgrade. This can happen
    # when user choses "Yes" when asked about overwritting the configuration file.
    if [ "${current_token}" = "SOLARWINDS_TOKEN" ]; then
        echodebug "Token was lost during upgrade or it was never set by user. Check if there is new one or there was one before upgrade and use it."
        if [ "${SOLARWINDS_TOKEN}" != "SOLARWINDS_TOKEN" ]; then
            echodebug "Adding new credentials into ${AGENT_CONF} configuration file."
            sed -i "s/SOLARWINDS_TOKEN/${SOLARWINDS_TOKEN}/g" "${AGENT_CONF}"
        elif [ -f "${UPGRADE_SAVE_TOKEN_FILE}" ]; then
            echodebug "Found token. Adding credentials back into ${AGENT_CONF} configuration file."
            sed -i "s/SOLARWINDS_TOKEN/${previous_token}/g" "${AGENT_CONF}"
        fi
    elif [ "${SOLARWINDS_TOKEN}" != "${previous_token}" ] && [ "${SOLARWINDS_TOKEN}" != "SOLARWINDS_TOKEN" ]; then
        echodebug "Adding new credentials into ${AGENT_CONF} configuration file."
        sed -i "s/${previous_token}/${SOLARWINDS_TOKEN}/g" "${AGENT_CONF}"
    fi
}

main() {
    echo -e " ${BC}======================${GC} ${AGENT_DISPLAY_NAME} Post Upgrade Install ${BC}=======================${EC}"
    AGENT_VERSION=$(get_package_version)
    echoblue "  This script will complete post upgrade installation of ${AGENT_DISPLAY_NAME}. Package version: ${AGENT_VERSION}."

    if [ -f "${AGENT_CONF}.rpmnew" ]; then
        AGENT_CONF="${AGENT_CONF}.rpmnew"
    fi

    set_swoken
    create_install_directories
    create_symlinks_tools
    check_token
    insert_appoptics_url

    create_daemon_user
    RETVAL=$?
    if [ ${RETVAL} -ne "${SUCCESS}" ]; then
        echo -e "${BC}Post upgrade installation of ${GC}${AGENT_DISPLAY_NAME}${BC} has ${RC}FAILED${BC}. Unable to create ${AGENT_USERNAME} system user.${EC}"
        failed ${RETVAL}
        exit ${RETVAL}
    fi

    set_daemon_user_rights
    RETVAL=$?
    if [ ${RETVAL} -ne "${SUCCESS}" ]; then
        echo -e "${BC}Post upgrade installation of ${GC}${AGENT_DISPLAY_NAME}${BC} has ${RC}FAILED${BC}. Unable to set access rights for ${AGENT_USERNAME} system user.${EC}"
        failed ${RETVAL}
        exit ${RETVAL}
    fi

    configure_log_rotation
    install_service
    RETVAL=$?
    if [ ${RETVAL} -ne "${SUCCESS}" ]; then
        echo -e "${BC}Post upgrade installation of ${GC}${AGENT_DISPLAY_NAME}${BC} has ${RC}FAILED${BC}. Unable to start service.${EC}"
        failed ${RETVAL}
        exit ${RETVAL}
    fi

    echo -e "${BC}Post upgrade installation of ${GC}${AGENT_DISPLAY_NAME}${BC} has ${GC}SUCCEEDED${EC}"

    echo -e " ${BC}======================${GC} ${AGENT_DISPLAY_NAME} Post Upgrade Install Completed ${BC}=======================${EC}"
    exit "${SUCCESS}"
}

main