# shellcheck disable=SC2148
VENDOR="${VENDOR:-solarwinds}"

DEFAULT_INSTALL_DIR="/opt/SolarWinds/Snap"
INSTALL_DIR=${INSTALL_DIR:-${DEFAULT_INSTALL_DIR}}
# shellcheck disable=SC2034
AGENT_CONF="${INSTALL_DIR}/etc/config.yaml"
LOGS_PLUGIN_CONFIG="${INSTALL_DIR}/etc/plugins.d/logs.yaml"
# shellcheck disable=SC2034
LOGS_PLUGIN_CONFIG_EXAMPLE="${LOGS_PLUGIN_CONFIG}.example"

# shellcheck disable=SC2034
AGENT_NAME="swisnapd"
# shellcheck disable=SC2034
DAEMON_NAME="swisnapd"
# shellcheck disable=SC2034
AGENT_DISPLAY_NAME="SolarWinds Snap Agent"

# shellcheck disable=SC2034
AGENT_TMP_DIR="/tmp/SolarWinds/Snap"
# shellcheck disable=SC2034
AGENT_LOG_DIR="/var/log/SolarWinds/Snap"
# shellcheck disable=SC2034
PID_DIR="/var/run/${VENDOR}"
# shellcheck disable=SC2034
SYMLINK_DIR=/usr/local/bin
# shellcheck disable=SC2034
LOG_ROTATION_CONF_DIR=/etc/logrotate.d

# shellcheck disable=SC2034
SYSTEMCTL_BIN=$(command -v systemctl 2>/dev/null)
# shellcheck disable=SC2034
CHKCONFIG_BIN=$(command -v chkconfig 2>/dev/null)
# shellcheck disable=SC2034
SERVICE_BIN=$(command -v service 2>/dev/null)
# shellcheck disable=SC2034
DPKG_QUERY_BIN=$(command -v dpkg-query 2>/dev/null)
# shellcheck disable=SC2034
LOGROTATE_BIN=$(command -v logrotate 2>/dev/null)
# shellcheck disable=SC2034
UPGRADE_SAVE_TOKEN_FILE="${INSTALL_DIR}/etc/upgrade.token.backup"
# shellcheck disable=SC2034
RM_CMD=/bin/rm

# Settings for the user used to run the daemon
# shellcheck disable=SC2034
AGENT_USERNAME=${VENDOR}
# shellcheck disable=SC2034
AGENT_GROUPNAME=${VENDOR}

# truth values
__IS_TRUE=1
__IS_FALSE=0

# When set to true, install the package but don't start the service
AGENT_INSTALL_ONLY=${AGENT_INSTALL_ONLY:-${APPOPTICS_INSTALL_ONLY:-${__IS_FALSE}}}

# Contains the current installation step of the post installer
# shellcheck disable=SC2034
__step=1

_ECHO_DEBUG=${_ECHO_DEBUG:-${__IS_FALSE}}
_SD_NO_COLOR=${_SD_NO_COLOR:-${__IS_TRUE}}

# shellcheck disable=SC2034
ROOT_DIR=$(pwd)

# Return error codes
readonly SUCCESS=0
readonly FAILURE=1
readonly FAILURE_SERVICE_NOT_RUNNING=2
readonly FAILURE_NOT_ENOUGH_RIGHTS=3
readonly FAILURE_UNABLE_TO_EXTRACT=4
readonly FAILURE_MISSING_EXECUTABLE=5
readonly FAILURE_CREATE_AGENT_GROUP=6
readonly FAILURE_CREATE_AGENT_USER=7
readonly FAILURE_SET_ACCESS_RIGHTS_AGENT_USER=8
_COLORS=${BS_COLORS:-$(tput colors 2>/dev/null || echo 0)}
detect_color_support() {
    # shellcheck disable=SC2181
    if [ $? -eq 0 ] && [ "$_COLORS" -gt 2 ] && [ "$_SD_NO_COLOR" -eq "$__IS_FALSE" ]; then
        RC="\033[1;31m"
        GC="\033[1;32m"
        BC="\033[1;34m"
        YC="\033[1;33m"
        EC="\033[0m"
    else
        RC=""
        GC=""
        BC=""
        YC=""
        EC=""
    fi
}
detect_color_support

# Override built-in echo for consistency. Has newline at end
echo() (
    fmt=%s end=\\n IFS=" "

    while [ $# -gt 1 ]; do
        case "$1" in
        [!-]* | -*[!ne]*) break ;;
        *ne* | *en*) fmt=%b end= ;;
        *n*) end= ;;
        *e*) fmt=%b ;;
        esac
        shift
    done

    # shellcheck disable=SC2059
    printf "$fmt$end" "$*"
)

echored() {
    printf "${RC}%s${EC}\n" "$*"
}

echogreen() {
    printf "${GC}%s${EC}\n" "$*"
}

echoblue() {
    printf "${BC}%s${EC}\n" "$*"
}

echoyellow() {
    printf "${YC}%s${EC}\n" "$*"
}
__increment_step() {
    __step=$((__step + 1))
}

exists() {
    command -v "$1" >/dev/null 2>&1
}

now() {
    date +'%Y-%m-%d-%H_%M_%S'
}

create_install_directories() {
    echodebug "Creating support directories ..."
    mkdir -p "${AGENT_TMP_DIR}"
    mkdir -p "${AGENT_LOG_DIR}"
    mkdir -p "${PID_DIR}"
    echodebug "directories created."
}

get_package_version() {
    local version=
    if [ -x "${DPKG_QUERY_BIN}" ]; then
        version="$(dpkg-query -W solarwinds-snap-agent)"
    else
        version="$(rpm -q solarwinds-snap-agent)"
    fi
    echo "${version}"
}

set_swoken() {
    if [ -n "${SOLARWINDS_TOKEN}" ]; then
        SOLARWINDS_TOKEN=${SOLARWINDS_TOKEN}
    elif [ -n "${APPOPTICS_TOKEN}" ]; then
        SOLARWINDS_TOKEN=${APPOPTICS_TOKEN}
    else
        SOLARWINDS_TOKEN="SOLARWINDS_TOKEN"
    fi
}

insert_appoptics_url() {
    sed -i "s,APPOPTICS_URL,\"${APPOPTICS_URL}\",g" "${AGENT_CONF}"
    sed -i "s,APPOPTICS_REPORT_URL,\"${APPOPTICS_REPORT_URL}\",g" "${AGENT_CONF}"
}
echoerror() {
    printf "${RC} * ERROR${EC}: %s\n" "$*" 1>&2
}

info() {
    printf "${GC} *  INFO${EC}: %s\n" "$*"
}

warning() {
    printf "${YC} *  WARN${EC}: %s\n" "$*"
}

echodebug() {
    if [ "$_ECHO_DEBUG" = "$__IS_TRUE" ]; then
        printf "${BC} * DEBUG${EC}: %s\n" "$*"
    fi
}

error() {
    error_code=${1}
    error_description=${2:-""}
    error_date_time="$(now)"
    echoerror "######################################################"
    echoerror "An error has occured during uninstallation of the agent."
    echoerror
    echoerror "Description: ${error_description}"
    echoerror "Error code : ${error_code}"
    echoerror "When       : ${error_date_time}"
    echoerror "######################################################"
}

failed() {
    error_code=${1}
    echo -e "${BC}===============${RC} FAILED ${BC}======================${EC}"
    echo -e "${BC}An error occurred during installation of ${RC}${AGENT_DISPLAY_NAME}${BC}.${EC}"
    echo -e "${BC}Please contact AppOptics customer support at ${RC}support@appoptics.com${BC}.${EC}"
    echo -e "${BC}Error code: ${RC}${error_code}${BC}.${EC}"
    echo -e "${BC}========================================================${EC}"
}
# before-remove

remove_service() {
    info "(${__step}) Checking if system uses systemd or SysV(/etc/init.d) to manage the services."
    __increment_step

    if [ -x "${SYSTEMCTL_BIN}" ]; then
        # Stop service
        info "(${__step}) Stopping ${AGENT_DISPLAY_NAME} service ..."
        __increment_step
        ${SYSTEMCTL_BIN} stop "${AGENT_NAME}"

        # Disable service to be executed at boot time
        info "(${__step}) Disabling ${AGENT_DISPLAY_NAME} service to be loaded at boot time ..."
        __increment_step
        ${SYSTEMCTL_BIN} disable "${AGENT_NAME}"
    else
        # Stop service
        info "(${__step}) Stopping ${AGENT_DISPLAY_NAME} service ..."
        __increment_step
        ${SERVICE_BIN} "${AGENT_NAME}" stop

        # Disable service to be executed at boot time
        info "(${__step}) Disabling ${AGENT_DISPLAY_NAME} service to be loaded at boot time ..."
        __increment_step
        if [ -x "${CHKCONFIG_BIN}" ]; then
            ${CHKCONFIG_BIN} "${AGENT_NAME}" off
        else
            /usr/sbin/update-rc.d -f "${AGENT_NAME}" remove
        fi

        # Remove service
        echodebug "Remove configuration for ${AGENT_DISPLAY_NAME} service ..."
        ${RM_CMD} -f "/etc/init.d/${AGENT_NAME}"
    fi
}

remove_daemon_user() {
    info "(${__step}) Removing user and group settings ..."
    __increment_step

    if getent passwd "${AGENT_USERNAME}" >/dev/null; then
        info "Deleting ${AGENT_USERNAME} system user ..."
        userdel "${AGENT_USERNAME}"
        RETVAL=$?
        if [ $RETVAL -ne 0 ]; then
            warning "Unable to delete ${AGENT_USERNAME} system user. Error code: ${RETVAL}."
        else
            info "${AGENT_USERNAME} deleted."
        fi
    fi

    if getent group "${AGENT_GROUPNAME}" >/dev/null; then
        info "Deleting ${AGENT_GROUPNAME} system group ..."
        groupdel "${AGENT_GROUPNAME}"
        RETVAL=$?
        if [ $RETVAL -ne 0 ]; then
            warning "Unable to delete ${AGENT_GROUPNAME} system group. Error code: ${RETVAL}."
        else
            info "${AGENT_GROUPNAME} deleted."
        fi
    fi

    echodebug "user and group settings removed."
}

main() {
    echo -e " ${BC}======================${GC} ${AGENT_DISPLAY_NAME} Uninstall ${BC}=======================${EC}"
    echoblue "  This script will complete uninstallation of ${AGENT_DISPLAY_NAME}."

    remove_service
    remove_daemon_user

    echo -e "${BC} Uninstall of ${GC}${AGENT_DISPLAY_NAME}${BC} has completed.${EC}"

    echo -e " ${BC}======================${GC} ${AGENT_DISPLAY_NAME} Uninstall Completed ${BC}=======================${EC}"
}

main